# Copyright 2026 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Any, Optional


@dataclass
class ResponsePart:
  """Represents a part of the LLM response.

  Attributes:
      text: The conversational text part. Can be an empty string.
      a2ui_json: The parsed A2UI JSON data, always a list of dictionaries if
        it contains A2UI messages. None if this part only contains trailing
        text.
  """

  text: str = ""
  a2ui_json: Optional[Any] = None
